package sqlite

import (
	"database/sql"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/repository"
)

type SqliteTodoRepository struct {
	db *sql.DB
}

func NewSqliteTodoRepository(db *sql.DB) *SqliteTodoRepository {
	return &SqliteTodoRepository{
		db: db,
	}
}

func (r *SqliteTodoRepository) GetById(id int64) (repository.TodoRow, error) {
	todo := repository.TodoRow{}

	row := r.db.QueryRow("SELECT * FROM todo WHERE id = ?;", id)
	err := row.Scan(&todo.Id, &todo.Name, &todo.Done)

	if err != nil {
		if err == sql.ErrNoRows {
			return todo, repository.ErrNotFound
		}

		return todo, err
	}

	return todo, nil
}

func (r *SqliteTodoRepository) Create(todo repository.TodoRow) (repository.TodoRow, error) {
	result, err := r.db.Exec("INSERT INTO todo (name, done) VALUES (?, ?)", todo.Name, todo.Done)

	if err != nil {
		return repository.TodoRow{}, err
	}

	id, err := result.LastInsertId()

	if err != nil {
		return repository.TodoRow{}, err
	}

	todo.Id = id

	return todo, nil
}

func (r *SqliteTodoRepository) Update(todo repository.TodoRow) error {
	result, err := r.db.Exec("UPDATE todo SET name = ?, done = ? WHERE id = ?", todo.Name, todo.Done, todo.Id)

	if err != nil {
		return err
	}

	rowsAffected, err := result.RowsAffected()

	if err != nil {
		return err
	}

	if rowsAffected == 0 {
		return repository.ErrNotFound
	}

	return nil
}

func (r *SqliteTodoRepository) Delete(id int64) error {
	result, err := r.db.Exec("DELETE FROM todo WHERE id = ?", id)

	if err != nil {
		return err
	}

	rowsAffected, err := result.RowsAffected()

	if err != nil {
		return err
	}

	if rowsAffected == 0 {
		return repository.ErrNotFound
	}

	return nil
}
